/*
 * @file:    EthDev_LM3S.c
 * @purpose: Ethernet Device Functions for LM3S6965
 * @version: V1.01
 * @date:    22. May 2009
 *----------------------------------------------------------------------------
 *
 * Copyright (C) 2009 ARM Limited. All rights reserved.
 *
 * ARM Limited (ARM) is supplying this software for use with Cortex-M3
 * processor based microcontrollers.  This file can be freely distributed
 * within development tools that are supporting such ARM based processors.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

#include "lm3s_cmsis.h"

#include "EthDev.h"
#include "EthDev_LM3S.h"

/*----------------------------------------------------------------------------
  Ethernet Device local functions
 *----------------------------------------------------------------------------*/
static int           EthDev_Init        (void);
static int           EthDev_UnInit      (void);
static int           EthDev_SetMCFilter (int NumHwAddr, unsigned char *pHwAddr);
static int           EthDev_TxFrame     (void *pData, int size);
static void          EthDev_Lock        (void);
static void          EthDev_UnLock      (void);
static EthDev_STATUS EthDev_LinkChk     (void);

static void *        EthDev_RxFrame     (int size);
static void          EthDev_RxFrameReady(int size);

static int           phy_Rd             (unsigned int PhyReg);
static int           phy_Wr             (unsigned int PhyReg, unsigned short Data);

/*----------------------------------------------------------------------------
  Ethernet Device IO Block
 *----------------------------------------------------------------------------*/

#define ETHDEV     eth0

EthDev_IOB ETHDEV = { 
                      EthDev_MODE_AUTO,
                      0,0,0,0,0,0,
                      EthDev_RxFrame,      /* Place holder, initialized by the user */
                      EthDev_RxFrameReady, /* Place holder, initialized by the user */

                      EthDev_Init,
                      EthDev_UnInit,
                      EthDev_SetMCFilter,
                      EthDev_TxFrame,
                      EthDev_Lock,
                      EthDev_UnLock,
                      EthDev_LinkChk
                    };

static EthDev_STATUS Status;

/*----------------------------------------------------------------------------
  Ethernet Device place holder function for user callback 
 *----------------------------------------------------------------------------*/
static void *EthDev_RxFrame (int size) {
  return (NULL);                         /* do nothing */
}


/*----------------------------------------------------------------------------
  Ethernet Device place holder function for user callback 
 *----------------------------------------------------------------------------*/
static void EthDev_RxFrameReady (int size) {
  ;                                      /* do nothing */
}


/*----------------------------------------------------------------------------
  Ethernet Device initialize
 *----------------------------------------------------------------------------*/
static int EthDev_Init (void) {
  int regv,tout;

  SYSCTL->RCGC2 |=  ((1UL<<30) | (1UL<<28));         /* enable clock for EPHY0, EMAC0 */

  SYSCTL->SRCR2 |=  ((1UL<<30) | (1UL<<28));         /* set reset for EPHY0, EMAC0 */
  for (tout = 0; tout < 0x2000; tout++) __NOP;       /* wait a while */
  SYSCTL->SRCR2 &= ~((1UL<<30) | (1UL<<28));         /* clear reset for EPHY0, EMAC0 */

  SYSCTL->RCGC2 |= (1<<5);                           /* enable clock for GPIOF */
  GPIOF->AFSEL  |= ((1<<1) | (1<<2));                /* PIN1, PIN2 alternate function */

  GPIOF->DR2R   |= ((1<<1) | (1<<2));                /* PIN1, PIN2 strength 2MA */
  GPIOF->DEN    |= ((1<<1) | (1<<2));                /* PIN1, PIN2 is push-pull */
  GPIOF->DIR    |= ((1<<1) | (1<<2));                /* PIN1, PIN2 is output */

  ETH->IM   = 0;                                     /* Disable all Ethernet Interrupts */
  ETH->RIS  = INT_ALL;
  ETH->MDV  = 10;                                    /* Set MII interface clock (max. 2.5MHz) */
//  ETH->MADR  = PHY_DEF_ADR;                        /* set the PHY address */

  phy_Wr (PHY_REG_CTRL, 0x8000);

  /* Wait for hardware reset to end. */
  for (tout = 0x20000; ; tout--) {
    regv = phy_Rd (PHY_REG_CTRL);
    if (regv < 0 || tout == 0) {
       /* Error */
       return (-1);
    }
    if (!(regv & 0x8000)) {
       /* Reset complete. */
       break;
    }
  }

  switch (ETHDEV.Mode) {
    case EthDev_MODE_AUTO:
      phy_Wr (PHY_REG_CTRL, PHY_AUTO_NEG);
      break;

    case EthDev_MODE_10M_FULL:
      phy_Wr (PHY_REG_CTRL, PHY_10M_FULL);
      break;

    case EthDev_MODE_10M_HALF:
      phy_Wr (PHY_REG_CTRL, PHY_10M_HALF);
      break;

    case EthDev_MODE_100M_FULL:
      phy_Wr (PHY_REG_CTRL, PHY_100M_FULL);
      break;

    case EthDev_MODE_100M_HALF:
      phy_Wr (PHY_REG_CTRL, PHY_100M_HALF);
      break;

    default:
       return (-1);
  }

  /* Configure TX/RX Control Registers, enable Multicast. */
  ETH->RCTL = RXCTL_BAD_CRC | RXCTL_RST_FIFO;

  ETH->TCTL = TXCTL_PAD_EN  | TXCTL_CRC_EN;

  /* Set the Ethernet MAC Address registers */
  ETH->IA1   = ((unsigned int)ETHDEV.HwAddr[5] << 8)  |
                (unsigned int)ETHDEV.HwAddr[4];
  ETH->IA0   = ((unsigned int)ETHDEV.HwAddr[3] << 24) |
               ((unsigned int)ETHDEV.HwAddr[2] << 16) |
               ((unsigned int)ETHDEV.HwAddr[1] << 8)  |
                (unsigned int)ETHDEV.HwAddr[0];

  ETH->RCTL |= RXCTL_RX_EN;                          /* Enable the Ethernet Controller */
  ETH->TCTL |= TXCTL_TX_EN;

  ETH->IM    = INT_RX;
  NVIC_EnableIRQ(Ethernet_IRQn);                     /* Enable ETH Interrupt */

  Status.Link = EthDev_LINK_DOWN;
  return (0);
}


/*----------------------------------------------------------------------------
  Ethernet Device Uninitialize
 *----------------------------------------------------------------------------*/
static int EthDev_UnInit (void) {

  ETH->IM = 0;                                       /* Disable all Ethernet Interrupts */
  ETH->RIS = INT_ALL;
  NVIC_DisableIRQ(Ethernet_IRQn);                    /* Disable ETH Interrupt */

  /* Power Down PHY */
  phy_Wr (PHY_REG_CTRL, 0x0800);

  return (0);
}


/*----------------------------------------------------------------------------
  Ethernet Device Interrupt Disable
 *----------------------------------------------------------------------------*/
static void EthDev_Lock (void) {
  NVIC_DisableIRQ(Ethernet_IRQn);                       /* Disable ETH Interrupt */
}


/*----------------------------------------------------------------------------
  Ethernet Device Interrupt Enable
 *----------------------------------------------------------------------------*/
static void EthDev_UnLock (void) {
  NVIC_EnableIRQ(Ethernet_IRQn);                     /* Enable ETH Interrupt */
}


/*----------------------------------------------------------------------------
  Set Multicast Filter Address to ethernet controller 
 *----------------------------------------------------------------------------*/
static int EthDev_SetMCFilter (int NumHwAddr, unsigned char *pHwAddr) {

  /* luminary profides no address filter nor hash table only enable /disable */
  if (NumHwAddr) {
    ETH->RCTL |=  RXCTL_MCAST_EN;                    /* Enable Multicast */
  } else {
    ETH->RCTL &= ~RXCTL_MCAST_EN;                    /* Disable Multicast */
  }

  return 0;
}


/*----------------------------------------------------------------------------
  Send frame to ENET ethernet controller 
 *----------------------------------------------------------------------------*/
static int EthDev_TxFrame (void *pData, int size) {
  unsigned short *sp;
  unsigned int len,val,tout;

  for (tout = TX_FRAME_TOUT;  ; tout--) {
    if ((ETH->TR & TXRQ_NEW_TX) == 0) {
      break;
    }
    if (tout == 0) {
      return (-1);                                   /* Error Timeout */
    }
  }

  sp   = (unsigned short *)pData;
  val  = size - 14;
  val |= (*sp++) << 16;
  ETH->DATA = val;

  for (len = (size + 1)>>2; len; sp += 2, len--) {   /* Copy frame data to EMAC packet buffers. */
    ETH->DATA = sp[0] | ((unsigned int)sp[1] << 16);
  }
  
  ETH->TR = TXRQ_NEW_TX;                             /* Start Transmitting */

  return (0);
}


/*----------------------------------------------------------------------------
  Ethernet Device Check the Link
 *----------------------------------------------------------------------------*/
static EthDev_STATUS EthDev_LinkChk (void) {
  int phyData;

  phyData = phy_Rd (PHY_REG_STAT);
  if (phyData < 0) {
    return (Status);
  }

  if (Status.Link == EthDev_LINK_DOWN) {
    if (phyData & 0x0004) {
      Status.Link = EthDev_LINK_UP;
      phyData = phy_Rd (PHY_REG_DIAG);
      if (phyData < 0) {
        return (Status);
      }
      Status.Duplex = (phyData & 0x0800) ? EthDev_DUPLEX_FULL : EthDev_DUPLEX_HALF;
      Status.Speed  = (phyData & 0x0400) ? EthDev_SPEED_100M  : EthDev_SPEED_10M;
      if (Status.Duplex == EthDev_DUPLEX_FULL) {
        /* Full duplex is enabled. */
        ETH->TCTL |= TXCTL_DUP_EN;
      }
    }
  }
  else {
    if ((phyData & 0x0004) == 0) {
      Status.Link   = EthDev_LINK_DOWN;
    }
  }
  return (Status);
}


/*----------------------------------------------------------------------------
  Ethernet Device Ethernet Controller Interrupt function.
  (not static because of Startup.s) 
 *----------------------------------------------------------------------------*/
void Ethernet_IRQHandler (void) {                    /* EthDev_ISR */
  unsigned int int_stat,val,RxLen,len;
  unsigned short *dp;

  while ((int_stat = ETH->RIS & INT_ALL) != 0) {
    ETH->RIS = int_stat;                             /* Acknowledge Interrupt */

    if (int_stat & INT_RX) {                         /* Receive interrupt */
      val   = ETH->DATA;
      RxLen = (val & 0xFFFF) - 6;
      /* DMA adds also 4-byte CRC and 2-byte 'size' to packet size. */
      if (RxLen > EthDev_MTU_SIZE || (int_stat & INT_RX_ERR)) {
        /* Invalid frame, ignore it and free buffer. */
        goto rel;
      }
      dp = ETHDEV.RxFrame (RxLen);
      if (dp != NULL) {
//        ETH->RCTL &= ~RXCTL_RX_EN;
        for (len = (RxLen + 3) >> 2; len; dp += 2, len--) {
          dp[0] = val >> 16;
          val = ETH->DATA;
          dp[1] = val & 0xFFFF;
        }

        ETHDEV.RxFrameReady (RxLen);                    /* call RxFrameReady() if provided */

        if ((RxLen - 1) & 0x02) {
          /* Drain the remaining 1 or 2 byte(s) of the CRC. */
          val = ETH->DATA;
        }
//        ETH->RCTL |= RXCTL_RX_EN;
      }
      else {
        /* In case of error discard the entire RX FIFO. */
rel:    ETH->RCTL &= ~RXCTL_RX_EN;
        ETH->RCTL |= RXCTL_RST_FIFO;
        ETH->RCTL |= RXCTL_RX_EN;
      }
    }
  }
}


/*----------------------------------------------------------------------------
  Write a data 'Data' to PHY register 'PhyReg'.  
 *----------------------------------------------------------------------------*/
static int phy_Wr (unsigned int PhyReg, unsigned short Data) {
  unsigned int timeOut;

  ETH->MTXD = Data;
  ETH->MCTL = (PhyReg << 3) | MCTL_WR | MCTL_START;

  for (timeOut = 0; timeOut < MII_WR_TOUT; timeOut++) { /* Wait until operation completed */
    if ((ETH->MCTL & MCTL_START) == 0) {
      return (0);
    }
  }

  return (-1);
}


/*----------------------------------------------------------------------------
  Read a PHY register 'PhyReg'. 
 *----------------------------------------------------------------------------*/
static int phy_Rd (unsigned int PhyReg) {
  unsigned int timeOut;

  ETH->MCTL = (PhyReg << 3) | MCTL_START;

  for (timeOut = 0; timeOut < MII_RD_TOUT; timeOut++) { /* Wait until operation completed */
    if ((ETH->MCTL & MCTL_START) == 0) {
      /* Return a 16-bit value. */
      return (ETH->MRXD & MRXD_MASK);
    }
  }

  return (-1);
}
