/*                                                                              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>       
 *                                                                              
 * This program is free software; you can redistribute it and/or modify         
 * it under the terms of the GNU General Public License as published by         
 * the Free Software Foundation; either version 3 of the License, or            
 * (at your option) any later version.                                          
 *                                                                              
 * This program is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License            
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        
 */

#include "adc.h"

void adc_on(adc_t * adc) {
//ADC on-off, start_convertion 
//CR2->ADON
  adc->CR2|=ADC_ADON;
}

void adc_off(adc_t * adc) {
//ADC on-off, start_convertion 
//CR2->ADON
  adc->CR2&=~ADC_ADON;
}

bool adc_adon_flag(adc_t * adc) {
  return(get_flagStatus(&(adc->CR2),ADC_ADON));
}

bool adc_eoc_flag(adc_t * adc) {
  return(get_flagStatus(&(adc->SR),ADC_EOC));
}
bool adc_jeoc_flag(adc_t * adc) {
  return(get_flagStatus(&(adc->SR),ADC_JEOC));
}

uint32_t adc_inj_data_read(adc_t * adc, adc_channel_t channel) {
  switch (channel) {
  case ADC_channel0:
    return((adc->JDR1)&0xFFFF);
  case ADC_channel1:
    return((adc->JDR2)&0xFFFF);
  case ADC_channel2:
    return((adc->JDR3)&0xFFFF);
  case ADC_channel3:
    return((adc->JDR4)&0xFFFF);
  default:
    return(0);
  }
}

uint32_t adc_reg_data_read(adc_t * adc) {
  return((adc->DR)&0xFFFF);
}

uint32_t adc_reg_data_adc2_read(adc_t * adc) {
  return(((adc->DR)&0xFFFF0000)>>16);
}


int adc_regular_channel_sequence(adc_t * adc, adc_sequence_t sequence, adc_channel_t channel){
//Select regular channels and sequence 
//SQRx
  const uint32_t bits=ADC_SQx_bit_mask;
  uint32_t shift_places=0;
  if((sequence>ADC_sequence16) || (sequence<ADC_sequence1)) { return (ERROR);}
  if((channel>ADC_channel17) || (channel<ADC_channel1)) { return (ERROR);}
  if (sequence<=ADC_sequence6) {
    shift_places=((uint32_t) sequence)*5;
    writeMultiBitValue(&(adc->SQR3),(uint8_t) channel,bits<<shift_places,shift_places);
  } else {
    if (sequence<=ADC_sequence12) {
      shift_places=((uint32_t) (sequence-ADC_sequence7))*5;
      writeMultiBitValue(&(adc->SQR2),(uint8_t) channel,bits<<shift_places,shift_places);
    } else {
      if (sequence<=ADC_sequence16) {
	shift_places=((uint32_t) (sequence-ADC_sequence13))*5;
	writeMultiBitValue(&(adc->SQR1),(uint8_t) channel,bits<<shift_places,shift_places);
      }
    }
  }
  return(SUCCESS);
}

int adc_regular_channel_count(adc_t * adc, uint32_t count) {
//Select quantity of regular channels
//SQR1->L
  if (count>=16) { return(ERROR);};
  writeMultiBitValue(&(adc->SQR1),(uint32_t) (count-1), ADC_L_bit_mask<<ADC_L_shift_places,ADC_L_shift_places);
  return(SUCCESS);
}


int adc_injected_channel_sequence(adc_t * adc, adc_sequence_t sequence, adc_channel_t channel){
//Select injected channels and sequence 
//JSQR
  if((sequence>ADC_sequence4) || (sequence<ADC_sequence1)) { return (ERROR);}
  if((channel>ADC_channel17) || (channel<ADC_channel1)) { return (ERROR);}

  uint32_t shift_places=0;
  const uint32_t bits=ADC_SQx_bit_mask;
  shift_places=((uint32_t) sequence)*5;

  writeMultiBitValue(&(adc->JSQR),(uint8_t) channel,bits<<shift_places,shift_places);
  return(SUCCESS);
}

int adc_injected_channel_count(adc_t * adc, uint32_t count) {
//Select quantity of injected channels
//JSQR->JL
  if (count>=4) { return(ERROR);};
  writeMultiBitValue(&(adc->JSQR),(uint32_t) (count-1), ADC_JL_bit_mask<<ADC_L_shift_places,ADC_JL_shift_places);
  return(SUCCESS);
}

//Analog input Vrefint is only available in ADC1

void adc_auto_trigger_enable(adc_t * adc) {
//Select between single and continuous conversion modes (a single scan scans all the channels selected in SQR (not sure if also the ones selected in JSQR) or not depending if the SCAN mode is selected)
//Conversion starts with CR2->ADON
//CR2->CONT
  adc->CR2|=ADC_CONT;
}
void adc_auto_trigger_disable(adc_t * adc) {
//Select between single and continuous conversion modes (a single scan scans all the channels selected in SQR (not sure if also the ones selected in JSQR) or not depending if the SCAN mode is selected)
//Conversion starts with CR2->ADON
//CR2->CONT
  adc->CR2&=~ADC_CONT;
}

void adc_scan_enable(adc_t * adc) {
//Scan mode (scans all the channels selected in SQRx or JSQR automatically). When it's disabled the channel scanned is selected in SQR3->SQ1
//CR1->SCAN
  adc->CR1|=ADC_SCAN;
}
void adc_scan_disable(adc_t * adc) {
//Scan mode (scans all the channels selected in SQRx or JSQR automatically). When it's disabled the channel scanned is selected in SQR3->SQ1
//CR1->SCAN
  adc->CR1&=~ADC_SCAN;
}

void adc_auto_injection_enable(adc_t * adc) {
//Select injection mode (triggered, auto-inject) (in auto-inject the injected channels are always converted after the regular channels)
//CR1->JAUTO, CR1->SCAN
  adc->CR1|=ADC_JAUTO|ADC_SCAN;
}

void adc_auto_injection_disable(adc_t * adc) {
//Select injection mode (triggered, auto-inject) (in auto-inject the injected channels are always converted after the regular channels)
//CR1->JAUTO, CR1->SCAN
  adc->CR1&=~ADC_JAUTO;
}

void adc_regular_split_mode_enable(adc_t *adc) {
//Select discontinuous mode for regular channels (convert a short sequence of n convertions which is part of the sequence of conversions selected in SQRx) (this should be called splitted mode to avoid confusion with the continuous mode above)
//CR1->DISCEN
  adc->CR1|=ADC_DISCEN;
}
void adc_regular_split_mode_disable(adc_t *adc) {
//Select discontinuous mode for regular channels (convert a short sequence of n convertions which is part of the sequence of conversions selected in SQRx) (this should be called splitted mode to avoid confusion with the continuous mode above)
//CR1->DISCEN
  adc->CR1&=~ADC_DISCEN;
}

void adc_injected_split_mode_enable(adc_t *adc) {
//Select discontinuous mode for injection channels (converts the sequence of JSQR channel by channel) (this should be called splitted mode to avoid confusion with the continuous mode above) (Not compatible with auto-injected mode) (Discontinuous mode should be avoided for both regular and injected channels at the same time)
//CR1->JDISCEN
  adc->CR1|=ADC_JDISCEN;
}
void adc_injected_split_mode_disable(adc_t *adc) {
//Select discontinuous mode for injection channels (converts the sequence of JSQR channel by channel) (this should be called splitted mode to avoid confusion with the continuous mode above) (Not compatible with auto-injected mode) (Discontinuous mode should be avoided for both regular and injected channels at the same time)
//CR1->JDISCEN
  adc->CR1&=~ADC_JDISCEN;
}

int adc_dual_mode(adc_t* adc, adc_dual_m_t mode) {
//Dual mode
//CR1->DUALMOD
  if (mode>ADC_m_alt_trig) {return(ERROR);};
  if (adc!=ADC1) {return(ERROR);};
  writeMultiBitValue(&(adc->CR1),(uint32_t) mode, ADC_DUALMOD_bit_mask<<ADC_DUALMOD_shift_places,ADC_DUALMOD_shift_places);
  return(SUCCESS);
}

//Routine to wait tSTAB time after we turn on the ADC

//Analog watchdog (later)

int adc_reg_split_mode_num(adc_t * adc, uint32_t count) {
//Write the number of conversions of discontinuous mode (splitted mode)
//CR1->DISCNUM
  if (count>8) {return(ERROR);};
  if (count<1) {return(ERROR);};
  writeMultiBitValue(&(adc->CR1),(uint32_t) (count-1), ADC_DISCNUM_bit_mask<<ADC_DISCNUM_shift_places,ADC_DISCNUM_shift_places);
  return(SUCCESS);
}

void adc_cal_start(adc_t* adc){
//Calibration
//CR2->CAL
  adc->CR2|=ADC_CAL;
}

bool adc_cal_status(adc_t* adc) {
  return(get_flagStatus(&(adc->CR2),ADC_CAL));
}

void adc_align_set(adc_t *adc, adc_align_t alignment) {
//Data alignment (left or right aligned). For injected channels only:  it is possible to to decrease the converted value by an offset given in JOFRx, so that the result can be negative.
//CR2->ALIGN, JOFRx
  set_bit(&(adc->CR2),ADC_ALIGN,alignment);
}

int adc_inj_align_offset(adc_t *adc, adc_channel_t channel, uint32_t offset) {
  if (offset>ADC_JOFFSETx_bit_mask) {return(ERROR);};
  switch (channel) {
  case ADC_channel1:
    writeMultiBitValue(&(adc->JOFR1),offset,ADC_JOFFSETx_bit_mask<<ADC_JOFFSETx_shift_places, ADC_JOFFSETx_shift_places);
    break;
  case ADC_channel2:
    writeMultiBitValue(&(adc->JOFR2),offset,ADC_JOFFSETx_bit_mask<<ADC_JOFFSETx_shift_places, ADC_JOFFSETx_shift_places);
    break;
  case ADC_channel3:
    writeMultiBitValue(&(adc->JOFR3),offset,ADC_JOFFSETx_bit_mask<<ADC_JOFFSETx_shift_places, ADC_JOFFSETx_shift_places);
    break;
  case ADC_channel4:
    writeMultiBitValue(&(adc->JOFR4),offset,ADC_JOFFSETx_bit_mask<<ADC_JOFFSETx_shift_places, ADC_JOFFSETx_shift_places);
    break;
  default:
    return(ERROR);
  }
  return(SUCCESS);
}


int adc_sampling_time(adc_t *adc, adc_channel_t channel, adc_smp_t sampling_time) {
//Sampling time per channel
//SMPR1,2->SMP
  uint32_t shift_places=0;
  if (sampling_time>ADC_smp_239_5) {return(ERROR);};
  if (channel>ADC_channel17) {return(ERROR);};
  if (channel<=ADC_channel9) {
    shift_places=((uint32_t) channel)*3;
    writeMultiBitValue(&(adc->SMPR2),sampling_time,ADC_SMPx_bit_mask<<shift_places,shift_places);
  } else {
    if (channel<=ADC_channel17) {
      shift_places=((uint32_t) (channel-ADC_channel10))*3;
      writeMultiBitValue(&(adc->SMPR1),sampling_time,ADC_SMPx_bit_mask<<shift_places,shift_places);
    }
  }
  return(SUCCESS);
}

void adc_reg_ext_trigger_conversion_enable(adc_t *adc) {
//Start conversion with an external event trigger (trigger sources, check table 45, 46)
//CR2->EXTTRIG, CR2->JEXTTRIG, CR2->EXTSEL, CR2->JEXTSEL 
  adc->CR2|=ADC_EXTTRIG;
}
void adc_reg_ext_trigger_conversion_disable(adc_t *adc) {
//Start conversion with an external event trigger (trigger sources, check table 45, 46)
//CR2->EXTTRIG, CR2->JEXTTRIG, CR2->EXTSEL, CR2->JEXTSEL 
  adc->CR2&=~ADC_EXTTRIG;
}
void adc_inj_ext_trigger_conversion_enable(adc_t *adc) {
//Start conversion with an external event trigger (trigger sources, check table 45, 46)
//CR2->EXTTRIG, CR2->JEXTTRIG, CR2->EXTSEL, CR2->JEXTSEL 
  adc->CR2|=ADC_JEXTTRIG;
}
void adc_inj_ext_trigger_conversion_disable(adc_t *adc) {
//Start conversion with an external event trigger (trigger sources, check table 45, 46)
//CR2->EXTTRIG, CR2->JEXTTRIG, CR2->EXTSEL, CR2->JEXTSEL 
  adc->CR2&=~ADC_JEXTTRIG;
}

int adc_reg_ext_trigger_select(adc_t *adc, adc_extsel_t selection) {
  if ((adc==ADC1) || (adc==ADC2)) {
    if (selection>ADC_EXTTRIG_adc12_swstart) {return(ERROR);};
    return(writeMultiBitValue(&(adc->CR2),(uint32_t) selection, ADC_EXTSEL_bit_mask<<ADC_EXTSEL_shift_places, ADC_EXTSEL_shift_places));
  } else { 
    if (adc==ADC3) {
      if ((selection>ADC_EXTTRIG_adc3_swstart) || (selection<ADC_EXTTRIG_adc3_tim3_cc1)) {return(ERROR);};
      return(writeMultiBitValue(&(adc->CR2),(uint32_t) (selection-ADC_EXTTRIG_adc3_tim3_cc1), ADC_EXTSEL_bit_mask<<ADC_EXTSEL_shift_places, ADC_EXTSEL_shift_places));
    } else {return(ERROR);}
  }
  return(SUCCESS);
}

int adc_inj_ext_trigger_select(adc_t *adc, adc_jextsel_t selection) {
  if ((adc==ADC1) || (adc==ADC2)) {
    if (selection>ADC_JEXTTRIG_adc12_jswstart) {return(ERROR);};
    return(writeMultiBitValue(&(adc->CR2),(uint32_t) selection, ADC_EXTSEL_bit_mask<<ADC_EXTSEL_shift_places, ADC_EXTSEL_shift_places));
  } else { 
    if (adc==ADC3) {
      if ((selection>ADC_JEXTTRIG_adc3_jswstart) || (selection<ADC_JEXTTRIG_adc3_tim1_trgo)) {return(ERROR);};
      return(writeMultiBitValue(&(adc->CR2),(uint32_t) (selection-ADC_JEXTTRIG_adc3_tim1_trgo), ADC_EXTSEL_bit_mask<<ADC_EXTSEL_shift_places, ADC_EXTSEL_shift_places));
    } else {return(ERROR);}
  }
  return(SUCCESS);
}

//Recommended sampling time for the temperatur sensor is 17.1us
//Formula: celsius=((V25-Vsense)/Avg_Slope)+25, Vsense=value taken for 25degrees

void adc_temp_vref_enable(adc_t * adc) {
//Enable both internal channels (Vtemp takes some time to stabilize, take a look in the specs)
//CR2->TSVREFE
  adc->CR2|=ADC_TSVREFE;
}
void adc_temp_vref_disable(adc_t * adc) {
//Enable both internal channels (Vtemp takes some time to stabilize, take a look in the specs)
//CR2->TSVREFE
  adc->CR2&=~ADC_TSVREFE;
}

//Interrupts ADC1 and ADC2 share the same interrupt vector!!!

//Enable interrupts: EOC, JEOC, AWD, End of conversion, injected end of conversion, analog watchdog status set
//CR1->EOCIE,JEOCIE,AWDIE
void adc_reg_eoc_int_enable(adc_t * adc) {
  adc->CR1|=ADC_EOCIE;
}
void adc_reg_eoc_int_disable(adc_t * adc) {
  adc->CR1&=~ADC_EOCIE;
}

void adc_inj_eoc_int_enable(adc_t * adc) {
  adc->CR1|=ADC_JEOCIE;
}
void adc_inj_eoc_int_disable(adc_t * adc) {
  adc->CR1&=~ADC_JEOCIE;
}

void adc_watchdog_int_enable(adc_t * adc) {
  adc->CR1|=ADC_AWDIE;
}
void adc_watchdog_int_disable(adc_t * adc) {
  adc->CR1&=~ADC_AWDIE;
}

void adc_set_conf(adc_t* adc, adc_conf_t conf) {
  adc_regular_channel_count(adc,conf.reg_ch_count);
  adc_injected_channel_count(adc,conf.inj_ch_count);
  if (conf.auto_trigger) {adc_auto_trigger_enable(adc);} else {adc_auto_trigger_disable(adc);};
  if (conf.scan) {adc_scan_enable(adc);} else {adc_scan_disable(adc);};
  if (conf.auto_injection) {adc_auto_injection_enable(adc);} else {adc_auto_injection_disable(adc);};
  if (conf.reg_split) {adc_regular_split_mode_enable(adc);} else {adc_regular_split_mode_disable(adc);};
  if (conf.inj_split) {adc_injected_split_mode_enable(adc);} else {adc_injected_split_mode_disable(adc);};
  adc_dual_mode(adc,conf.dual_mod);
  adc_reg_split_mode_num(adc,conf.reg_split_count);
  adc_align_set(adc,conf.alignment);
  //uint32_t align_offset_ch1;
  //uint32_t align_offset_ch2;
  //uint32_t align_offset_ch3;
  //uint32_t align_offset_ch4;
  if (conf.reg_ext_trigger) {adc_reg_ext_trigger_conversion_enable(adc);} else {adc_reg_ext_trigger_conversion_disable(adc);};
  if (conf.inj_ext_trigger) {adc_inj_ext_trigger_conversion_enable(adc);} else {adc_inj_ext_trigger_conversion_disable(adc);};
  adc_reg_ext_trigger_select(adc,conf.reg_exttrig_sel);
  adc_inj_ext_trigger_select(adc,conf.inj_exttrig_sel);
  if (conf.temp_vref) {adc_temp_vref_enable(adc);} else {adc_temp_vref_disable(adc);};
  if (conf.reg_eoc_int) {adc_reg_eoc_int_enable(adc);} else {adc_reg_eoc_int_disable(adc);};
  if (conf.inj_eoc_int) {adc_inj_eoc_int_enable(adc);} else {adc_inj_eoc_int_disable(adc);};
  if (conf.watchdot_int) {adc_watchdog_int_enable(adc);} else {adc_watchdog_int_disable(adc);};
}

adc_conf_t adc(void) {
  adc_conf_t adc_conf;
  adc_conf.reg_ch_count=1;
  adc_conf.inj_ch_count=1;
  adc_conf.auto_trigger=false;
  adc_conf.scan=false;
  adc_conf.auto_injection=false;
  adc_conf.reg_split=false;
  adc_conf.inj_split=false;
  adc_conf.dual_mod=ADC_m_independent;
  adc_conf.reg_split_count=1;
  adc_conf.alignment=false;
  adc_conf.reg_ext_trigger=false;
  adc_conf.inj_ext_trigger=false;
  adc_conf.reg_exttrig_sel=ADC_EXTTRIG_adc12_tim1_cc1;
  adc_conf.inj_exttrig_sel=ADC_JEXTTRIG_adc12_tim1_trgo;
  adc_conf.temp_vref=false;
  adc_conf.reg_eoc_int=false;
  adc_conf.inj_eoc_int=false;
  adc_conf.watchdot_int=false;
  return(adc_conf);
}
