/*              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *                
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *                                                  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.                                       
 *                      
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "rcc.h"
#include <stdbool.h>
#include "utils.h"
#include "types.h"

//todo:
//add clock dependecy check
//add ready osc check for the clock dependency check
//add getClockFrequency(CLK) for any clock signal.

//reset reason rutine RCC_CSR
//soft reset: SYSRESETREQ Cortex application int and reset control register + RCC_BDCR
//standby and stop reset
//SYSCLK sources HSI HSE PLLCLK
//PLL sources HSIDIV2 HSEPLL
//RTCCLK sources HSEDIV128, LSE, LSI
//MCO sources PLLCLKDIV2, HSI, HSE, SYSCLK
//USBCLK sources PLLCLKDIV1_5, PLLCLK
//AHBCLK sources SYSCLK divided from 1,2 ...512
//APB1CLK div 1,2,4,8,16
//TIM(2-7)CLK div 1,2
//APB2CLK div 1,2,4,8,16
//TIM(1,8)CLK div 1,2
//ADCCLK div 2,4,6,8
//Functions to give the current freq value of all clock signals from above and also
// HSI, USBCLK, I2S3CLK, I2S2CLK, SDIOCLK, FSMCCLK, HCLK(AHBBUSCLK,CORECLK,MEMCLK,DMACLK), SYSTICKCLK, FCLK, PLLINCLK, PLLCLK, SYSCLK, AHBCLK, APB1CLK, PCLK1, TIM(2-7)CLK, HSECLK, APB2CLK, PCLK2, TIM(1,8)CLK, LSECLK, RTCCLK, ADCCLK, HCLKDIV2CLK(SDIOAHBCLK), LSICLK, IWDGCLK, MCOCLK
//HSE sources selection: EXTCLK, EXTOSC (HSEBYP, HSEON: RCC_CR)
//HSE stability detection function. HSERDY RCC_CR
//HSE on/off (HSEON RCC_CR)
//HSI calibration HSICAL[7:0] RCC_CR, HSITRIM[4:0] RCC_CR
//HSI stability detection function HSIRDY RCC_CR
//HSI on/off
//LSE sources selection: EXTCLK, EXTOSC: LSEBYP LSEON RCC_BDCR
//LSI on/off LSION RCC_CSR
//LSI stability LSIRDY RCC_CSR
//CSS enable (clock security system)

clks_t rcc_sysclk_source_status(void){
  uint8_t status_bits;
  clks_t status;
  status_bits=((RCC->CFGR)&(SYSCLKSTATUS_bit_mask<<SYSCLKSTATUS_shift_places)>>SYSCLKSTATUS_shift_places);
  switch (status_bits) {
  case SYSCLKSTATUS_hsi:
    status=HSICLK;
    break;
  case SYSCLKSTATUS_hse:
    status=HSECLK;
    break;
  case SYSCLKSTATUS_pll:
    status=PLLCLK;
    break;
  default:
    break;
  }
  return(status);
}

void rcc_backupDomReset(void) {
    RCC->BDCR|=BDRST;
}

int rcc_periReset(peri_t peri) {
    __RW uint32_t * rcc_reg;
    uint8_t shift_places;
    if ((peri<=APB2_LAST) && (peri>=APB2_FIRST)) {
        rcc_reg=&(RCC->APB2RSTR);
        shift_places=((uint8_t) (peri-APB2_FIRST));
    } else {
        if ((peri<=APB1_LAST) && (peri>=APB1_FIRST)) {
            rcc_reg=&(RCC->APB1RSTR);
            shift_places=((uint8_t) (peri-APB1_FIRST));
        } else {
            return(ERROR);
        }
    }
    *rcc_reg=(((uint32_t) 0x1)<<shift_places);
    return(SUCCESS);
}

int rcc_periClkEnable(peri_t peri) {
    __RW uint32_t * rcc_reg;
    uint8_t shift_places;
    if ((peri<=APB2_LAST) && (peri>=APB2_FIRST)) {
        rcc_reg=&(RCC->APB2ENR);
        shift_places=((uint8_t) (peri-APB2_FIRST));
    } else {
        if ((peri<=APB1_LAST) && (peri>=APB1_FIRST)) {
            rcc_reg=&(RCC->APB1ENR);
            shift_places=((uint8_t) (peri-APB1_FIRST));
        } else {
            if ((peri<=AHB_LAST) && (peri>=AHB_FIRST)) {
                rcc_reg=&(RCC->AHBENR);
                shift_places=((uint8_t) (peri-AHB_FIRST));
            } else {
                if (peri==RTC) {
                    rcc_reg=&(RCC->BDCR);
                    shift_places=(uint8_t) RTCEN;
                } else {
                    return(ERROR);
                }
            }
        }
    }
    *rcc_reg|=(((uint32_t) 0x1)<<shift_places);
    return(SUCCESS);
}

int rcc_periClkDisable(peri_t peri) {
    __RW uint32_t * rcc_reg;
    uint8_t shift_places;
    if ((peri<=APB2_LAST) && (peri>=APB2_FIRST)) {
        rcc_reg=&(RCC->APB2ENR);
        shift_places=((uint8_t) (peri-APB2_FIRST));
    } else {
        if ((peri<=APB1_LAST) && (peri>=APB1_FIRST)) {
            rcc_reg=&(RCC->APB1ENR);
            shift_places=((uint8_t) (peri-APB1_FIRST));
        } else {
            if ((peri<=AHB_LAST) && (peri>=AHB_FIRST)) {
                rcc_reg=&(RCC->AHBENR);
                shift_places=((uint8_t) (peri-AHB_FIRST));
            } else {
                if (peri==RTC) {
                    rcc_reg=&(RCC->BDCR);
                    shift_places=(uint8_t) RTCEN;
                } else {
                    return(ERROR);
                }
            }
        }
    }
    *rcc_reg&=~(((uint32_t) 0x1)<<shift_places);
    return(SUCCESS);
}

int rcc_oscReadyIntClear(osc_t osc) {
    //set the ready clear flags to zero if I don't want to touch them
    uint32_t oscReadyIntbit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscReadyIntbit=PLLRDYC;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscReadyIntbit=HSERDYC;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscReadyIntbit=HSIRDYC;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscReadyIntbit=LSERDYC;
        rcc_reg=&(RCC->CIR);
        break;
    case LSI:
        oscReadyIntbit=LSIRDYC;
        rcc_reg=&(RCC->CIR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)|=oscReadyIntbit;
    return(SUCCESS);

}
int rcc_oscReadyIntEnable(osc_t osc){
    //set the ready clear flags to zero if I don't want to touch them
    uint32_t oscReadyIntEnablebit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscReadyIntEnablebit=PLLRDYIE;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscReadyIntEnablebit=HSERDYIE;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscReadyIntEnablebit=HSIRDYIE;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscReadyIntEnablebit=LSERDYIE;
        rcc_reg=&(RCC->CIR);
        break;
    case LSI:
        oscReadyIntEnablebit=LSIRDYIE;
        rcc_reg=&(RCC->CIR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)|=oscReadyIntEnablebit;
    return(SUCCESS);
}

int rcc_oscReadyIntDisable(osc_t osc){
    //set the ready clear flags to zero if I don't want to touch them
    uint32_t oscReadyIntEnablebit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscReadyIntEnablebit=PLLRDYIE;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscReadyIntEnablebit=HSERDYIE;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscReadyIntEnablebit=HSIRDYIE;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscReadyIntEnablebit=LSERDYIE;
        rcc_reg=&(RCC->CIR);
        break;
    case LSI:
        oscReadyIntEnablebit=LSIRDYIE;
        rcc_reg=&(RCC->CIR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)&=~oscReadyIntEnablebit;
    return(SUCCESS);
}

bool rcc_oscReadyIntFlag(osc_t osc){
    uint32_t oscReadyIntFlagbit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscReadyIntFlagbit=PLLRDYF;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscReadyIntFlagbit=HSERDYF;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscReadyIntFlagbit=HSIRDYF;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscReadyIntFlagbit=LSERDYF;
        rcc_reg=&(RCC->CIR);
        break;
    case LSI:
        oscReadyIntFlagbit=LSIRDYF;
        rcc_reg=&(RCC->CIR);
        break;
    default:
        return(false);
    }
    return(get_flagStatus(rcc_reg,oscReadyIntFlagbit));
}

void rcc_cssIntClear(void){
    //todo: set the ready clear flags to zero if I don't want to touch them
    RCC->CIR|=CSSC;
}
bool rcc_cssIntFlag(void) {
    return(get_flagStatus(&(RCC->CIR),CSSF));
}

bool rcc_is_oscReady( osc_t osc) {
    uint32_t oscReadybit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscReadybit=PLLRDY;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscReadybit=HSERDY;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscReadybit=HSIRDY;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscReadybit=LSERDY;
        rcc_reg=&(RCC->BDCR);
        break;
    case LSI:
        oscReadybit=LSIRDY;
        rcc_reg=&(RCC->CSR);
        break;
    default:
        return(false);
    }
    return(get_flagStatus(rcc_reg,oscReadybit));
}

int rcc_oscOn( osc_t osc) {
    uint32_t oscOnbit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscOnbit=PLLON;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscOnbit=HSEON;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscOnbit=HSION;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscOnbit=LSEON;
        rcc_reg=&(RCC->BDCR);
        break;
    case LSI:
        oscOnbit=LSION;
        rcc_reg=&(RCC->CSR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)|=oscOnbit;
    return(SUCCESS);
}

int rcc_oscOff( osc_t osc) {
    uint32_t oscOffbit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case PLL:
        oscOffbit=PLLON;
        rcc_reg=&(RCC->CR);
        break;
    case HSE:
        oscOffbit=HSEON;
        rcc_reg=&(RCC->CR);
        break;
    case HSI:
        oscOffbit=HSION;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscOffbit=LSEON;
        rcc_reg=&(RCC->BDCR);
        break;
    case LSI:
        oscOffbit=LSION;
        rcc_reg=&(RCC->CSR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)&=~oscOffbit;
    return(SUCCESS);
}


void rcc_cssEnable(void) {
    RCC->CR|=CSSON;
}

void rcc_cssDisable(void) {
    RCC->CR&=~CSSON;
}

int rcc_oscBypEnable( osc_t osc) {
    uint32_t oscBypbit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case HSE:
        oscBypbit=HSEBYP;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscBypbit=LSEBYP;
        rcc_reg=&(RCC->BDCR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)|=oscBypbit;
    return(SUCCESS);
}

int rcc_oscBypDisable( osc_t osc) {
    uint32_t oscBypbit;
    __RW uint32_t * rcc_reg;
    switch(osc) {
    case HSE:
        oscBypbit=HSEBYP;
        rcc_reg=&(RCC->CR);
        break;
    case LSE:
        oscBypbit=LSEBYP;
        rcc_reg=&(RCC->BDCR);
        break;
    default:
        return(ERROR);
    }
    (*rcc_reg)&=~oscBypbit;
    return(SUCCESS);
}

uint8_t rcc_HSICalRead(void) {
    return((RCC->CR&HSICAL_bits) >> HSICAL_shift_places);
}

uint8_t rcc_HSITrimRead(void) {
    return((RCC->CR&HSITRIM_bits) >> HSITRIM_shift_places);
}

int rcc_HSITrimWrite( uint8_t trimvalue) {
    return(writeMultiBitValue(&(RCC->CR),trimvalue,HSITRIM_bits,HSITRIM_shift_places));
}

int rcc_MCOSourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,MCOSEL,NOCLK);
        CASE(outval,MCOSEL,SYSCLK);
        CASE(outval,MCOSEL,HSICLK);
        CASE(outval,MCOSEL,HSECLK);
        CASE(outval,MCOSEL,PLLCLKDIV2);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,MCOSEL_bits,MCOSEL_shift_places));
}


int rcc_USBPRESourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,USBPRE,PLLCLK);
        CASE(outval,USBPRE,PLLCLKDIV1_5);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,USBPRE_bits,USBPRE_shift_places));
}

int rcc_PLLFreqScaleSel( uint8_t factor) {
    if((factor>17) || (factor<2)) {
        return(ERROR);
    }
    uint32_t outfactor=factor-2;
    return(writeMultiBitValue(&(RCC->CFGR),outfactor,PLLMUL_bits,PLLMUL_shift_places));
}

int rcc_PLLSourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,PLLSEL,HSICLKDIV2);
        CASE(outval,PLLSEL,HSECLK);
        CASE(outval,PLLSEL,HSECLKDIV2);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,PLLSEL_bits,PLLSEL_shift_places));
}

int rcc_ADCPRESourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,ADCPRE,PLCLK2DIV2);
        CASE(outval,ADCPRE,PLCLK2DIV4);
        CASE(outval,ADCPRE,PLCLK2DIV6);
        CASE(outval,ADCPRE,PLCLK2DIV8);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,ADCPRE_bits,ADCPRE_shift_places));
}

int rcc_APB2PRESourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,APB2PRE,HCLK);
        CASE(outval,APB2PRE,HCLKDIV2);
        CASE(outval,APB2PRE,HCLKDIV4);
        CASE(outval,APB2PRE,HCLKDIV8);
        CASE(outval,APB2PRE,HCLKDIV16);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,APB2PRE_bits,APB2PRE_shift_places));
}

int rcc_APB1PRESourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,APB1PRE,HCLK);
        CASE(outval,APB1PRE,HCLKDIV2);
        CASE(outval,APB1PRE,HCLKDIV4);
        CASE(outval,APB1PRE,HCLKDIV8);
        CASE(outval,APB1PRE,HCLKDIV16);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,APB1PRE_bits,APB1PRE_shift_places));
}

int rcc_AHBPRESourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,AHBPRE,SYSCLK);
        CASE(outval,AHBPRE,SYSCLKDIV2);
        CASE(outval,AHBPRE,SYSCLKDIV4);
        CASE(outval,AHBPRE,SYSCLKDIV8);
        CASE(outval,AHBPRE,SYSCLKDIV16);
        CASE(outval,AHBPRE,SYSCLKDIV64);
        CASE(outval,AHBPRE,SYSCLKDIV128);
        CASE(outval,AHBPRE,SYSCLKDIV256);
        CASE(outval,AHBPRE,SYSCLKDIV512);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,AHBPRE_bits,AHBPRE_shift_places));
}


/*uint8_t rcc_SYSCLKSourceRead(void) {
    return((RCC->CR&SYSCLKSTATUS_bits) >> SYSCLKSTATUS_shift_places);
    }*/


int rcc_SYSCLKSourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,SYSCLKSEL,HSICLK);
        CASE(outval,SYSCLKSEL,HSECLK);
        CASE(outval,SYSCLKSEL,PLLCLK);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,SYSCLKSEL_bits,SYSCLKSEL_shift_places));
}

int rcc_RTCCLKSourceSel( clks_t clk) {
    uint32_t outval;
    switch (clk) {
        CASE(outval,RTCSEL,NOCLK);
        CASE(outval,RTCSEL,LSECLK);
        CASE(outval,RTCSEL,LSICLK);
        CASE(outval,RTCSEL,HSECLKDIV128);
    default:
        return(ERROR);
    }
    return(writeMultiBitValue(&(RCC->CFGR),outval,RTCSEL_bits,RTCSEL_shift_places));
}

last_reset_t rcc_lastReset(void) {
    switch (RCC->CSR&RESET_FLAG_BITS) {
    case PINRSTF:
        return(PINRST);
    case PORRSTF:
        return(PORRST);
    case SFTRSTF:
        return(SFTRST);
    case IWDGRSTF:
        return(IWDGRST);
    case WWDGRSTF:
        return(WWDGRST);
    case LPWRRSTF:
        return(LPWRRST);
    default:
        return(RST_ERROR_REASON);
    }
}

void rcc_clearLastReset(void) {
    RCC->CSR|=RMVF;
}
/*
void rcc_enable_APB2_peripherals( uint32_t bits) {
    RCC->APB2ENR|=bits;
}

void rcc_disable_APB2_peripherals( uint32_t bits) {
    RCC->APB2ENR&=~bits;
}
*/
