/*              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *                
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *                                                  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.                                       
 *                      
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tim.h"
//#include <stdint.h>

//interrupt related functions
void tim_all_int_disable(tim_t* tim) {
  tim->DIER=0;
}

void tim_update_int_enable(tim_t* tim) { //Enables generation of an interrupt with every event
  tim->DIER|=TIM_UIE;
}

void tim_update_int_disable(tim_t* tim) { 
  tim->DIER&=~TIM_UIE;
}

void tim_COM_int_enable(tim_t* tim) { //Enables generation of an interrupt with COM
  tim->DIER|=TIM_COMIE;
}

void tim_COM_int_disable(tim_t* tim) { 
  tim->DIER&=~TIM_COMIE;
}

void tim_trigger_int_enable(tim_t* tim) { //Enables generation of an interrupt with trigger
  tim->DIER|=TIM_TIE;
}

void tim_trigger_int_disable(tim_t* tim) { 
  tim->DIER&=~TIM_TIE;
}

//Event interrupt related functions
void tim_update_event_enable(tim_t* tim) { //(useful for enabling/disabling update while the software is writing new data to ARR, PSC or CCRx)
  tim->CR1&=~TIM_UDIS;
}

void tim_update_event_disable(tim_t* tim) { //Use this before writing new data to ARR, PSC, CCRx
  tim->CR1|=TIM_UDIS;
}
//UDIS CR1
void tim_update_event_source_select(tim_t* tim, tim_upevent_src_t upevent_source);  //0)counter over/underflow, setting UG, update gen through slave mode controller 1) counter over/underflow
//URS CR1

bool tim_update_event_interrupt_flag(tim_t* tim, bool clear) { //an update event ocurred
//UIF SR
  bool out;
  set_bit(&(tim->SR), TIM_UIF, ~clear);
  out=get_flagStatus(&(tim->SR),TIM_UIF);
  return(out);
}

//Time base unit related functions
//uint16_t tim_tbu_read(tim_t* tim, tim_tbur_t reg); //read from a time base unit register CNT, PSC, ARR and RCR
//CNT, PSC, ARR, RCR

//void tim_tbu_write(tim_t* tim, tim_tbur_t reg, uint16_t data) { //write to a time base unit register
//}
//CNT, PSC, ARR, RCR.  Always write 0 to the 8 high bits

void tim_arr_buffered_enable(tim_t* tim); //whether ARR register is buffered or not
void tim_arr_buffered_disable(tim_t* tim);
//ARPE (CR1)

void tim_clock_enable(tim_t* tim) { //clock (counter) enable, is recommended to generate an UG (force update event) before start counting.
  tim_force_update_event(tim);
  tim->CR1|=TIM_CEN;
}
void tim_clock_disable(tim_t* tim) {
  //Counter disable
  tim->CR1&=~TIM_CEN;
}
//CEN (CR1)

void tim_force_update_event(tim_t* tim) { //generates an update event.
  tim->EGR|=TIM_UG;
}
//UG (EGR)

void tim_count_mode_sel(tim_t* tim, tim_count_mode_t count_mode, tim_count_dir_t init_count_dir) { //selects the counting mode and the initial counting direction. For center-aligned mode, it sets the DIR bit only if the counter is disabled (CEN=OFF)
  writeMultiBitValue(&(tim->CR1),(uint32_t) count_mode ,TIM_CMS_bits,TIM_CMS_shift_places);
  if (count_mode!=TIM_CNT_M_EDGE) {
    if ((tim->CR1 & TIM_CEN) != TIM_CEN) {
      set_bit(&(tim->CR1),TIM_DIR,init_count_dir);
    }
  } else {
    set_bit(&(tim->CR1),TIM_DIR,init_count_dir);
  } 
}
//CMS, DIR (CR1)

//Counter stops after update event
void tim_one_pulse_mode_enable(tim_t* tim) {
  tim->CR1|=TIM_OPM;
}
void tim_one_pulse_mode_disable(tim_t* tim) {
  tim->CR1&=~TIM_OPM;
}
//OPM (CR1)

//Clock selection and trigger related functions
//selects the clock mode (encoder mode1 , encoder mode2, encoder mode3, external clock mode1 (trigger inputs), external clock mode2, internal clock mode)
//selects between the trigger mode types (none, reset, gated and trigger)
//Not all combinations work
//ECE (check documentation warnings), SMS (SMCR)
//This function must be rewritten.
int tim_clock_trigger_mode_select(tim_t* tim, tim_clk_mode_t clock_mode, tim_trg_mode_t trigger_mode) {
  if (trigger_mode!=TIM_TRG_M_NONE) {
    if ((clock_mode!=TIM_CLK_M_INT) || (clock_mode!=TIM_CLK_M_EXT2)) {
      return(ERROR);
    }
    writeMultiBitValue(&(tim->SMCR),((uint32_t) trigger_mode),TIM_SMS_bits,TIM_SMS_shift_places);
    if (clock_mode==TIM_CLK_M_EXT2) {
      tim->SMCR|=TIM_ECE;
    } else {
      tim->SMCR&=~TIM_ECE;
    }
  } else {
    if (clock_mode!=TIM_CLK_M_EXT2) {
      writeMultiBitValue(&(tim->SMCR),((uint32_t) clock_mode),TIM_SMS_bits,TIM_SMS_shift_places);
      tim->SMCR&=~TIM_ECE;
    } else {
      writeMultiBitValue(&(tim->SMCR),((uint32_t) TIM_CLK_M_INT),TIM_SMS_bits,TIM_SMS_shift_places);   
      tim->SMCR|=TIM_ECE;
    }
  }
  return(SUCCESS);
}

void tim_trigger_source_select(tim_t* tim, tim_trg_src_t trigger_src); //selects the trigger signal source (internal triggers(ITR0-3), TI1 edge detector (TI1F_ED), Filtered timer input(1,2) (TI(1,2)FP(1,2)), external trigger input (ETRF)
//Change this only if SMS=000
//TS (SMCR)

void tim_external_trigger_input_conf(tim_t* tim, tim_etr_t etr); //Configures the external trigger input circuitry
//ETP, ETPS, ETF (SMCR) tim_etr_t(struct)

//Capture/compare channels related functions
//void tim_ccr_write(tim_t* tim, tim_channel_t channel, uint16_t data); //writes data to the CCRy

//void tim_ccr_read(tim_t* tim, tim_channel_t channel, uint16_t data); //reads data from the CCRy

void tim_cc_channel_event_gen(tim_t* tim, tim_channel_t channel); //generates an event (possible INT and DMA) in the specific channel. In capture mode it generates a capture.
//CCyG (EGR)

//input capture related
int tim_ic_conf(tim_t* tim, tim_channel_t channel, tim_ic_t ic_conf); //Configures the capture input channel
//Not all input sources are possible. It depends on the channel. check CCyS.
//returns error if Trigger input is not internal (check TS in trigger unit) when selecting TRC as input_sel and/or filter value bigger than FFFF, also returns an error if we are not in input compare mode (CCMRx.CCyS)

//General functions for all output compare channels
void tim_oc_main_output_enable(tim_t* tim) { // Main enable for the compare outputs
  tim->BDTR|=TIM_MOE;
}
void tim_oc_main_output_disable(tim_t* tim) {
  tim->BDTR&=~TIM_MOE;
}
//MOE (BDTR)

bool tim_oc_main_output_disabled(tim_t* tim) { // Returns if the outputs are disabled (by software or by ext break input
  return(get_flagStatus(&(tim->BDTR),TIM_MOE));
}
//MOE (BDTR)

void tim_oc_main_output_autoenable_enable(tim_t* tim); //automatically enables outputs at the next update event
void tim_oc_main_output_autoenable_disable(tim_t* tim);
//AOE (BDTR)

void tim_oc_break_set_polarity(tim_t* tim, bool polarity);
//BKP (BDTR)
void tim_oc_break_enable(tim_t* tim); //enable break inputs (BRK and BRK_ACTH)
void tim_oc_break_disable(tim_t* tim); 
//BKE (BDTR)

//OSSR and OSSI not clear yet.
void tim_oc_breaked_outputs_disable(tim_t* tim) { //outputs are disable
}
void tim_oc_breaked_outputs_force_idle_state(tim_t* tim); //outputs are forced to the programmed idle state (see OIS (CR2))
//OSSI (BDTR)
void tim_oc_not_breaked_outputs_disable(tim_t* tim); //outputs are disable
void tim_oc_not_breaked_outputs_force_idle_state(tim_t* tim); //outputs are forced to the programmed idle state (see OIS (CR2))
//OSSR (BDTR)

//Functions for specific output compare channels

int tim_oc_conf(tim_t* tim, tim_channel_t channel, tim_oc_t conf) { //configures the output compare channel
//here we must set CCMR.CCyS to output (00)
  //Setting the channel to output mode
  switch (channel) {
  case TIM_CHANNEL1:
    writeMultiBitValue(&(tim->CCMR1),((uint32_t) TIM_CCyS_OUTPUT),TIM_CC1S_bits,TIM_CC1S_shift_places);
    writeMultiBitValue(&(tim->CCMR1),((uint32_t) conf.oc_mode),TIM_OC1M_bits,TIM_OC1M_shift_places);
    switch (conf.oc_output_mode) {
    case OC_OUTPUT_M_DEAD_TIME_ACTIVE:
      tim->CCER|=(TIM_CC1E|TIM_CC1NE);
      break;
    case OC_OUTPUT_M_OC_ACTIVE:
      tim->CCER|=TIM_CC1E;
      tim->CCER&=~TIM_CC1NE;
      break;
    case OC_OUTPUT_M_OCN_ACTIVE:
      tim->CCER|=TIM_CC1NE;
      tim->CCER&=~TIM_CC1E;
      break;
    case OC_OUTPUT_M_OC_OCN_INACTIVE:
      tim->CCER&=~(TIM_CC1E|TIM_CC1NE);
      break;
    default:
      return(ERROR);
    }
    set_bit(&(tim->CCER),TIM_CC1P,!conf.oc_polarity);
    set_bit(&(tim->CCER),TIM_CC1NP,!conf.ocn_polarity);
    set_bit(&(tim->CR2),TIM_OIS1,conf.oc_idle_output_state);
    set_bit(&(tim->CR2),TIM_OIS1N,conf.ocn_idle_output_state);
    set_bit(&(tim->CCMR1),TIM_OC1CE,conf.oc_ocref_clear_on_ETRF);
    set_bit(&(tim->CCMR1),TIM_OC1PE,conf.oc_preload_ccr);
    set_bit(&(tim->CCMR1),TIM_OC1FE,conf.oc_fast_enable);
    set_bit(&(tim->DIER),TIM_CC1IE,conf.enable_int);
    set_bit(&(tim->DIER),TIM_CC1DE,conf.enable_dma);
    break;
  case TIM_CHANNEL2:
    writeMultiBitValue(&(tim->CCMR1),TIM_CCyS_OUTPUT,TIM_CC2S_bits,TIM_CC2S_shift_places);
    writeMultiBitValue(&(tim->CCMR1),conf.oc_mode,TIM_OC2M_bits,TIM_OC2M_shift_places);
    switch (conf.oc_output_mode) {
    case OC_OUTPUT_M_DEAD_TIME_ACTIVE:
      tim->CCER|=(TIM_CC2E|TIM_CC2NE);
      break;
    case OC_OUTPUT_M_OC_ACTIVE:
      tim->CCER|=TIM_CC2E;
      tim->CCER&=~TIM_CC2NE;
      break;
    case OC_OUTPUT_M_OCN_ACTIVE:
      tim->CCER|=TIM_CC2NE;
      tim->CCER&=~TIM_CC2E;
      break;
    case OC_OUTPUT_M_OC_OCN_INACTIVE:
      tim->CCER&=~(TIM_CC2E|TIM_CC2NE);
      break;
    default:
      return(ERROR);
    }
    set_bit(&(tim->CCER),TIM_CC2P,!conf.oc_polarity);
    set_bit(&(tim->CCER),TIM_CC2NP,!conf.ocn_polarity);
    set_bit(&(tim->CR2),TIM_OIS2,conf.oc_idle_output_state);
    set_bit(&(tim->CR2),TIM_OIS2N,conf.ocn_idle_output_state);
    set_bit(&(tim->CCMR1),TIM_OC2CE,conf.oc_ocref_clear_on_ETRF);
    set_bit(&(tim->CCMR1),TIM_OC2PE,conf.oc_preload_ccr);
    set_bit(&(tim->CCMR1),TIM_OC2FE,conf.oc_fast_enable);
    set_bit(&(tim->DIER),TIM_CC2IE,conf.enable_int);
    set_bit(&(tim->DIER),TIM_CC2DE,conf.enable_dma);
    break;
  case TIM_CHANNEL3:
    writeMultiBitValue(&(tim->CCMR2),TIM_CCyS_OUTPUT,TIM_CC3S_bits,TIM_CC3S_shift_places);
    writeMultiBitValue(&(tim->CCMR2),conf.oc_mode,TIM_OC3M_bits,TIM_OC3M_shift_places);
    switch (conf.oc_output_mode) {
    case OC_OUTPUT_M_DEAD_TIME_ACTIVE:
      tim->CCER|=(TIM_CC3E|TIM_CC3NE);
      break;
    case OC_OUTPUT_M_OC_ACTIVE:
      tim->CCER|=TIM_CC3E;
      tim->CCER&=~TIM_CC3NE;
      break;
    case OC_OUTPUT_M_OCN_ACTIVE:
      tim->CCER|=TIM_CC3NE;
      tim->CCER&=~TIM_CC3E;
      break;
    case OC_OUTPUT_M_OC_OCN_INACTIVE:
      tim->CCER&=~(TIM_CC3E|TIM_CC3NE);
      break;
    default:
      return(ERROR);
    }
    set_bit(&(tim->CCER),TIM_CC3P,!conf.oc_polarity);
    set_bit(&(tim->CCER),TIM_CC3NP,!conf.ocn_polarity);
    set_bit(&(tim->CR2),TIM_OIS3,conf.oc_idle_output_state);
    set_bit(&(tim->CR2),TIM_OIS3N,conf.ocn_idle_output_state);
    set_bit(&(tim->CCMR2),TIM_OC3CE,conf.oc_ocref_clear_on_ETRF);
    set_bit(&(tim->CCMR2),TIM_OC3PE,conf.oc_preload_ccr);
    set_bit(&(tim->CCMR2),TIM_OC3FE,conf.oc_fast_enable);
    set_bit(&(tim->DIER),TIM_CC3IE,conf.enable_int);
    set_bit(&(tim->DIER),TIM_CC3DE,conf.enable_dma);
    break;
  case TIM_CHANNEL4:
    writeMultiBitValue(&(tim->CCMR2),TIM_CCyS_OUTPUT,TIM_CC4S_bits,TIM_CC4S_shift_places);
    writeMultiBitValue(&(tim->CCMR2),conf.oc_mode,TIM_OC4M_bits,TIM_OC4M_shift_places);
    switch (conf.oc_output_mode) {
    case OC_OUTPUT_M_OC_ACTIVE:
      tim->CCER|=TIM_CC4E;
      break;
    case OC_OUTPUT_M_OC_INACTIVE:
      tim->CCER&=~TIM_CC4E;
      break;
    default:
      return(ERROR);
    }
    set_bit(&(tim->CCER),TIM_CC4P,!conf.oc_polarity);
    set_bit(&(tim->CR2),TIM_OIS4,conf.oc_idle_output_state);
    set_bit(&(tim->CCMR2),TIM_OC4CE,conf.oc_ocref_clear_on_ETRF);
    set_bit(&(tim->CCMR2),TIM_OC4PE,conf.oc_preload_ccr);
    set_bit(&(tim->CCMR2),TIM_OC4FE,conf.oc_fast_enable);
    set_bit(&(tim->DIER),TIM_CC4IE,conf.enable_int);
    set_bit(&(tim->DIER),TIM_CC4DE,conf.enable_dma);
    break;
  default:
    return(ERROR);
  }
  return(SUCCESS);
};

void tim_test_forced_output_mode(tim_t* timer,tim_channel_t channel) {
  tim_clock_disable(timer);
  tim_oc_t conf;
  conf.oc_mode=OC_MODE_FORCE_ACTIVE;
  conf.oc_output_mode=OC_OUTPUT_M_OC_ACTIVE;
  conf.oc_polarity=high;
  conf.ocn_polarity=high;
  conf.oc_idle_output_state=low;
  conf.ocn_idle_output_state=low;
  conf.oc_ocref_clear_on_ETRF=false;
  conf.oc_preload_ccr=true;
  conf.oc_fast_enable=false;
  conf.enable_int=false;
  conf.enable_dma=false;
  tim_oc_conf(timer,channel,conf);
  tim_update_event_enable(timer);
  tim_force_update_event(timer);
  tim_one_pulse_mode_disable(timer);
  tim_clock_enable(timer);
  tim_oc_main_output_enable(timer);
  tim_force_update_event(timer);
};

void tim_test_output_compare_mode(tim_t* timer, tim_channel_t channel) {

  //tbu unit config
  tim_clock_disable(timer); //Disables counter
  tim_oc_main_output_disable(timer); //Disables compare outputs
  tim_count_mode_sel(timer,TIM_CNT_M_EDGE,TIM_CNT_DIR_UP); //Selects counting mode
  tim_clock_trigger_mode_select(timer,TIM_CLK_M_INT,TIM_TRG_M_NONE); //No special triggering, count source from internal clock.

  tim_one_pulse_mode_disable(timer); //disable one pulse mode
  tim_update_event_disable(timer); //disable update events (for writing new stuff to data registers
  tim_tbu_write(timer,TIM_CNT,0); //write 0 to time counter
  tim_tbu_write(timer,TIM_PSC,4); //clock prescaler
  tim_tbu_write(timer,TIM_ARR,200); //Auto reload register

  //Capture compare configuration
  tim_ccr_write(timer,channel,100); //Capture compare register

  //Output compare conf
  tim_oc_t conf;
  conf.oc_mode=OC_MODE_PWM1; //Output active as counter smaller than compare value
  conf.oc_output_mode=OC_OUTPUT_M_DEAD_TIME_ACTIVE; 
  conf.oc_polarity=high;
  conf.ocn_polarity=high;
  conf.oc_idle_output_state=low;
  conf.ocn_idle_output_state=low;
  conf.oc_ocref_clear_on_ETRF=false;
  conf.oc_preload_ccr=false;
  conf.oc_fast_enable=false;
  conf.enable_int=true;
  conf.enable_dma=false;
  tim_oc_conf(timer,channel,conf);

  tim_update_event_enable(timer);
  tim_force_update_event(timer);
  //tim_oc_main_output_enable(timer);
  tim_clock_enable(timer);
  //  tim_force_update_event(timer);
  /*  timer->CR1=0x0000;
  timer->CR2=0x0000;
  timer->SMCR=0x0000;
  timer->DIER=0x0000;
  timer->EGR=0x0000;
  timer->CCMR1=0x0020;
  timer->CCMR2=0x0000;
  timer->CCER=0x0001;
  timer->CNT=0x0000;
  timer->PSC=0x0100;
  timer->ARR=0x0200;
  timer->RCR=0x0002;
  timer->CCR1=0x0050; //probar con 50
  timer->BDTR=0x8000; // *
  timer->DCR=0x0000;
  timer->DMAR=0x0000;
  timer->EGR=0x0001; // update generation, when ARR, CNT, CCR1 RCR are ready then this.
  timer->CR1=0x0001;
  *///timer->EGR=0x0001; // update generation, when ARR, CNT, CCR1 RCR are ready then this.*/
};


void tim_test_PWM1_mode(tim_t* timer, tim_channel_t channel) {

  //tbu unit config
  tim_clock_disable(timer); //Disables counter
  tim_all_int_disable(timer); //Disable all interrupts
  tim_oc_main_output_disable(timer); //Disables compare outputs
  tim_count_mode_sel(timer,TIM_CNT_M_CENTER_OUT_INT_UPDOWN,TIM_CNT_DIR_UP); //Selects counting mode
  tim_clock_trigger_mode_select(timer,TIM_CLK_M_INT,TIM_TRG_M_NONE); //No special triggering, count source from internal clock.

  tim_one_pulse_mode_disable(timer); //disable one pulse mode
  tim_update_event_disable(timer); //disable update events (for writing new stuff to data registers
  tim_tbu_write(timer,TIM_CNT,0); //write 0 to time counter
  tim_tbu_write(timer,TIM_PSC,4); //clock prescaler
  tim_tbu_write(timer,TIM_ARR,200); //Auto reload register

  //Capture compare configuration
  tim_ccr_write(timer,channel,100); //Capture compare register

  //Output compare conf
  tim_oc_t conf;
  conf.oc_mode=OC_MODE_PWM1; //Output active as counter smaller than compare value
  conf.oc_output_mode=OC_OUTPUT_M_DEAD_TIME_ACTIVE; 
  conf.oc_polarity=high;
  conf.ocn_polarity=high;
  conf.oc_idle_output_state=low;
  conf.ocn_idle_output_state=low;
  conf.oc_ocref_clear_on_ETRF=false;
  conf.oc_preload_ccr=false;
  conf.oc_fast_enable=false;
  conf.enable_int=true;
  conf.enable_dma=false;
  tim_oc_conf(timer,channel,conf);

  tim_update_event_enable(timer);
  tim_force_update_event(timer);
  tim_oc_main_output_enable(timer);
  tim_clock_enable(timer);
  tim_update_int_enable(timer);
};

void tim_tbu_conf(tim_t* timer, tim_count_mode_t count_mode, tim_count_dir_t init_count_dir, tim_clk_mode_t clock_mode, tim_trg_mode_t trigger_mode, uint16_t init_counter_value, uint16_t clock_prescaler, uint16_t auto_reload_value) {
  //tbu unit config
  tim_count_mode_sel(timer,count_mode, init_count_dir); //Selects counting mode
  tim_clock_trigger_mode_select(timer,clock_mode, trigger_mode); //No special triggering, count source from internal clock.

  tim_one_pulse_mode_disable(timer); //disable one pulse mode
  tim_update_event_disable(timer); //disable update events (for writing new stuff to data registers
  tim_tbu_write(timer,TIM_CNT,init_counter_value); //write 0 to time counter
  tim_tbu_write(timer,TIM_PSC,clock_prescaler); //clock prescaler
  tim_tbu_write(timer,TIM_ARR,auto_reload_value); //Auto reload register
};
 
void tim_PWM1_oc_conf(tim_t* timer, tim_channel_t channel, bool enable_interrupt) {
  //Output compare conf
  tim_oc_t conf;
  conf.oc_mode=OC_MODE_PWM1; //Output active as counter smaller than compare value
  conf.oc_output_mode=OC_OUTPUT_M_DEAD_TIME_ACTIVE; 
  conf.oc_polarity=high;
  conf.ocn_polarity=high;
  conf.oc_idle_output_state=low;
  conf.ocn_idle_output_state=low;
  conf.oc_ocref_clear_on_ETRF=false;
  conf.oc_preload_ccr=false;
  conf.oc_fast_enable=false;
  conf.enable_int=enable_interrupt;
  conf.enable_dma=false;
  tim_oc_conf(timer,channel,conf);
};

