/*              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *                
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *                                                  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.                                       
 *                      
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USART_H
#define USART_H

#include "mem_map.h"
#include "types.h"
#include "utils.h"
#include <stdint.h>

typedef struct {
  __RW uint32_t SR; //
  __RW uint32_t DR; //
  __RW uint32_t BRR; //
  __RW uint32_t CR1; //
  __RW uint32_t CR2; //
  __RW uint32_t CR3; //
  __RW uint32_t GTPR; //
} usart_t;

#define USART1 ((usart_t *) USART1_BASE)

/*SR*/
#define USART_TC bit6
#define USART_RXNE bit5

/*BRR*/
#define USART_DIV_MANTISSA_shift_places 4
#define USART_DIV_MANTISSA_bits (bit4|bit5|bit6|bit7|bit8|bit9|bit10|bit11|bit12|bit13|bit14|bit15)
#define USART_DIV_FRACTION_shift_places 0
#define USART_DIV_FRACTION_bits (bit0|bit1|bit2|bit3)

/*CR1*/
#define USART_UE bit13
#define USART_M bit12
#define USART_TE bit3
#define USART_RE bit2
#define USART_RXNEIE bit5
#define USART_TCIE bit6

/*CR2*/
#define USART_STOP_shift_places 12
#define USART_STOP_bits (bit12|bit13)

static inline void usart_enable(usart_t* usart) {
  usart->CR1|=USART_UE;
}
static inline void usart_disable(usart_t* usart) {
  usart->CR1&=~USART_UE;
}

typedef enum {
  USART_8_N=false,
  USART_9_1=true
} usart_word_len_t;

static inline void usart_word_len_set(usart_t* usart,usart_word_len_t word_len) {
  set_bit(&(usart->CR1),USART_M, word_len);
}

typedef enum {
  USART_1_STOP=0,
  USART_0_5_STOP=1,
  USART_2_STOP=2,
  USART_1_5_STOP=3
} usart_stop_bits_t;

static inline void usart_stop_bits_set(usart_t* usart,usart_stop_bits_t stop_bits) {
  writeMultiBitValue(&(usart->CR2),(uint32_t) stop_bits,USART_STOP_bits,USART_STOP_shift_places);
}

static inline void usart_transmit_enable(usart_t* usart) {
  usart->CR1|=USART_TE;
}

static inline void usart_transmit_disable(usart_t* usart) {
  usart->CR1&=~USART_TE;
}

static inline void usart_receive_enable(usart_t* usart) {
  usart->CR1|=USART_RE;
}

static inline void usart_receive_disable(usart_t* usart) {
  usart->CR1&=~USART_RE;
}

static inline void usart_baud_rate_write(usart_t* usart,uint16_t mantissa, uint8_t fraction) {
  writeMultiBitValue(&(usart->BRR),mantissa,USART_DIV_MANTISSA_bits,USART_DIV_MANTISSA_shift_places);
  writeMultiBitValue(&(usart->BRR),fraction,USART_DIV_FRACTION_bits,USART_DIV_FRACTION_shift_places);
}

static inline int usart_usartdiv_set(usart_t* usart,float usartdiv) {
  const float max=4095.9375;
  if (usartdiv>max) { return(ERROR);} 
  uint16_t num_int;
  uint8_t num_remainder;
  num_int=(uint16_t) usartdiv;
  num_remainder=(uint8_t) (16.0*(usartdiv-((float) num_int)));
  usart_baud_rate_write(usart,num_int,num_remainder);
  return(SUCCESS);
}

static inline int usart_baud_rate_set(usart_t* usart, uint32_t baudrate, uint32_t pclk) {
  float usartdiv;
  usartdiv=pclk/(16*baudrate);
  return(usart_usartdiv_set(usart, usartdiv));
}

static inline void usart_data_write(usart_t* usart, uint8_t data) {
  usart->DR=data;
}

static inline uint8_t usart_data_read(usart_t* usart) {
  return(usart->DR);
}

static inline void usart_receive_register_full_int_enable(usart_t* usart) {
  usart->CR1|=USART_RXNEIE;
}

static inline void usart_receive_register_full_int_disable(usart_t* usart) {
  usart->CR1&=~USART_RXNEIE;
}

static inline void usart_transmit_complete_int_enable(usart_t* usart) {
  usart->CR1|=USART_TCIE;
}

static inline void usart_transmit_complete_int_disable(usart_t* usart) {
  usart->CR1&=~USART_TCIE;
}

static inline bool usart_transmit_complete_flag(usart_t* usart) {
  if (((usart->SR)&USART_TC)==USART_TC) {
    return(true);
  } else {
    return(false);
  }
}

static inline void usart_ndata_write(usart_t* usart, char* data) {
  int i=0;
  while (data[i]!=0) {
    usart_data_write(usart,data[i]);
    i++;
    while (!usart_transmit_complete_flag(usart)) {
    }
  }
}

static inline bool usart_receive_register_full_flag(usart_t* usart) {
  if (((usart->SR)&USART_RXNE)==USART_RXNE) {
    return(true);
  } else {
    return(false);
  }
}

void itoa(int n, char s[], int b);
void ftoa(float n, char s[], int decimal_places);
void reverse_str(char s[],int n);

#endif /*USART_H*/
